@@ -31,12 +31,16 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "blowfish.h"
 
 #include "macros.h"
 
-#include <assert.h>
-
 /* Initial keysetup state */
 static const struct blowfish_ctx
 initial_ctx =
@@ -226,8 +230,6 @@ initial_ctx =
     0x082EFA98,0xEC4E6C89,0x452821E6,0x38D01377,0xBE5466CF,0x34E90C6C,
     0xC0AC29B7,0xC97C50DD,0x3F84D5B5,0xB5470917,0x9216D5D9,0x8979FB1B
   },
-  /* Initial value, reset after key is checked. */
-  BLOWFISH_WEAK_KEY
 };
 
 /* It's unfortunate to have to pick the bytes apart in the round
@@ -243,7 +245,7 @@ initial_ctx =
 #define R(c, l, r, i)  do { l ^= c->p[i]; r ^= F(c, l); } while(0)
 
 static void
-encrypt(struct blowfish_ctx *bc, uint32_t *ret_xl, uint32_t *ret_xr)
+encrypt(const struct blowfish_ctx *bc, uint32_t *ret_xl, uint32_t *ret_xr)
 {
   uint32_t xl, xr;
 
@@ -274,7 +276,7 @@ encrypt(struct blowfish_ctx *bc, uint32_t *ret_xl, uint32_t *ret_xr)
 }
 
 static void
-decrypt(struct blowfish_ctx *bc, uint32_t *ret_xl, uint32_t *ret_xr )
+decrypt(const struct blowfish_ctx *bc, uint32_t *ret_xl, uint32_t *ret_xr )
 {
   uint32_t xl, xr;
 
@@ -309,13 +311,11 @@ decrypt(struct blowfish_ctx *bc, uint32_t *ret_xl, uint32_t *ret_xr )
 #undef R
 
 void
-blowfish_encrypt(struct blowfish_ctx *bc, unsigned length,
+blowfish_encrypt(const struct blowfish_ctx *bc, unsigned length,
                  uint8_t *outbuf, const uint8_t *inbuf)
 {
     uint32_t d1, d2;
 
-    assert(!bc->status);
-
     FOR_BLOCKS(length, outbuf, inbuf, BLOWFISH_BLOCK_SIZE)
       {
 	d1 = READ_UINT32(inbuf);
@@ -330,13 +330,11 @@ blowfish_encrypt(struct blowfish_ctx *bc, unsigned length,
 
 
 void
-blowfish_decrypt(struct blowfish_ctx *bc, unsigned length,
+blowfish_decrypt(const struct blowfish_ctx *bc, unsigned length,
                  uint8_t *outbuf, const uint8_t *inbuf )
 {
     uint32_t d1, d2;
 
-    assert(!bc->status);
-
     FOR_BLOCKS(length, outbuf, inbuf, BLOWFISH_BLOCK_SIZE)
       {
 	d1 = READ_UINT32(inbuf);
@@ -356,15 +354,6 @@ blowfish_set_key(struct blowfish_ctx *ctx,
     int i, j;
     uint32_t data, datal, datar;
 
-#if 0
-    static int initialized = 0;
-
-    if( !initialized ) {
-	initialized = 1;
-	assert(selftest());
-    }
-#endif
-
     *ctx = initial_ctx;
 
     for(i=j=0; i < _BLOWFISH_ROUNDS+2; i++ )
@@ -416,6 +405,5 @@ blowfish_set_key(struct blowfish_ctx *ctx,
 	    (ctx->s[2][i] == ctx->s[2][j]) || (ctx->s[3][i] == ctx->s[3][j]) )
 	  return 0;
     
-    ctx->status = BLOWFISH_OK;
     return 1;
 }
