@@ -23,6 +23,10 @@
  * MA 02111-1307, USA.
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "md5-compat.h"
 
 void
@@ -40,7 +44,5 @@ MD5Update(MD5_CTX *ctx, const unsigned char *data, unsigned int length)
 void
 MD5Final(unsigned char *out, MD5_CTX *ctx)
 {
-  md5_final(ctx);
   md5_digest(ctx, MD5_DIGEST_SIZE, out);
-  md5_init(ctx);
 }
