@@ -74,6 +74,17 @@ struct nettle_cipher
   (nettle_crypt_func *) name##_decrypt,			\
 }
 
+#define _NETTLE_CIPHER_SEP_SET_KEY(name, NAME, keysize) {\
+  #name #keysize,					\
+  sizeof(struct name##_ctx),				\
+  NAME##_BLOCK_SIZE,					\
+  keysize / 8,						\
+  (nettle_set_key_func *) name##_set_encrypt_key,	\
+  (nettle_set_key_func *) name##_set_decrypt_key,	\
+  (nettle_crypt_func *) name##_crypt,			\
+  (nettle_crypt_func *) name##_crypt,			\
+}
+
 #define _NETTLE_CIPHER_FIX(name, NAME, keysize) {	\
   #name,						\
   sizeof(struct name##_ctx),				\
@@ -90,6 +101,11 @@ extern const struct nettle_cipher nettle_aes192;
 extern const struct nettle_cipher nettle_aes256;
 
 extern const struct nettle_cipher nettle_arcfour128;
+
+extern const struct nettle_cipher nettle_camellia128;
+extern const struct nettle_cipher nettle_camellia192;
+extern const struct nettle_cipher nettle_camellia256;
+
 extern const struct nettle_cipher nettle_cast128;
 
 extern const struct nettle_cipher nettle_serpent128;
@@ -137,7 +153,10 @@ extern const struct nettle_hash nettle_md2;
 extern const struct nettle_hash nettle_md4;
 extern const struct nettle_hash nettle_md5;
 extern const struct nettle_hash nettle_sha1;
+extern const struct nettle_hash nettle_sha224;
 extern const struct nettle_hash nettle_sha256;
+extern const struct nettle_hash nettle_sha384;
+extern const struct nettle_hash nettle_sha512;
 
 struct nettle_armor
 {
