@@ -386,6 +386,24 @@ test_hash_large(const struct nettle_hash *hash,
   free(data);
 }
 
+void
+test_mac(const struct nettle_mac *mac,
+	 unsigned key_length, const uint8_t *key,
+	 unsigned msg_length, const uint8_t *msg,
+	 const uint8_t *digest)
+{
+  void *ctx = xalloc(mac->context_size);
+  uint8_t *buffer = xalloc(mac->digest_size);
+
+  mac->set_key(ctx, key_length, key);
+  mac->update(ctx, msg_length, msg);
+  mac->digest(ctx, mac->digest_size, buffer);
+  ASSERT(MEMEQ(mac->digest_size, digest, buffer));
+
+  free(ctx);
+  free(buffer);
+}
+
 void
 test_armor(const struct nettle_armor *armor,
            unsigned data_length,
@@ -450,9 +468,9 @@ mpz_togglebit (mpz_t x, unsigned long int bit)
 #endif /* HAVE_LIBGMP */
 
 #if WITH_HOGWEED
-#define SIGN(key, hash, msg, signature) do {	\
-  hash##_update(&hash, LDATA(msg));		\
-  rsa_##hash##_sign(key, &hash, signature);	\
+#define SIGN(key, hash, msg, signature) do {		\
+  hash##_update(&hash, LDATA(msg));			\
+  ASSERT(rsa_##hash##_sign(key, &hash, signature));	\
 } while(0)
 
 #define VERIFY(key, hash, msg, signature) (	\
@@ -600,8 +618,8 @@ test_rsa_md5(struct rsa_public_key *pub,
 
 void
 test_rsa_sha1(struct rsa_public_key *pub,
-	     struct rsa_private_key *key,
-	     mpz_t expected)
+	      struct rsa_private_key *key,
+	      mpz_t expected)
 {
   struct sha1_ctx sha1;
   mpz_t signature;
@@ -643,8 +661,8 @@ test_rsa_sha1(struct rsa_public_key *pub,
 
 void
 test_rsa_sha256(struct rsa_public_key *pub,
-	     struct rsa_private_key *key,
-	     mpz_t expected)
+		struct rsa_private_key *key,
+		mpz_t expected)
 {
   struct sha256_ctx sha256;
   mpz_t signature;
@@ -684,6 +702,49 @@ test_rsa_sha256(struct rsa_public_key *pub,
   mpz_clear(signature);
 }
 
+void
+test_rsa_sha512(struct rsa_public_key *pub,
+		struct rsa_private_key *key,
+		mpz_t expected)
+{
+  struct sha512_ctx sha512;
+  mpz_t signature;
+
+  sha512_init(&sha512);
+  mpz_init(signature);
+
+  SIGN(key, sha512, "The magic words are squeamish ossifrage", signature);
+
+  if (verbose)
+    {
+      fprintf(stderr, "rsa-sha512 signature: ");
+      mpz_out_str(stderr, 16, signature);
+      fprintf(stderr, "\n");
+    }
+
+  if (mpz_cmp(signature, expected))
+    FAIL();
+  
+  /* Try bad data */
+  if (VERIFY(pub, sha512,
+	     "The magick words are squeamish ossifrage", signature))
+    FAIL();
+
+  /* Try correct data */
+  if (!VERIFY(pub, sha512,
+	      "The magic words are squeamish ossifrage", signature))
+    FAIL();
+
+  /* Try bad signature */
+  mpz_togglebit(signature, 17);
+
+  if (VERIFY(pub, sha512,
+	     "The magic words are squeamish ossifrage", signature))
+    FAIL();
+
+  mpz_clear(signature);
+}
+
 #undef SIGN
 #undef VERIFY
 
@@ -758,14 +819,15 @@ test_rsa_key(struct rsa_public_key *pub,
   mpz_clear(tmp); mpz_clear(phi);
 }
 
-#define DSA_VERIFY(key, hash, msg, signature) (	\
-  sha1_update(hash, LDATA(msg)),		\
-  dsa_verify(key, hash, signature)		\
-)
+/* Requires that the context is named like the hash algorithm. */
+#define DSA_VERIFY(key, hash, msg, signature)	\
+  (hash##_update(&hash, LDATA(msg)),		\
+   dsa_##hash##_verify(key, &hash, signature))
 
 void
-test_dsa(const struct dsa_public_key *pub,
-	 const struct dsa_private_key *key)
+test_dsa160(const struct dsa_public_key *pub,
+	    const struct dsa_private_key *key,
+	    const struct dsa_signature *expected)
 {
   struct sha1_ctx sha1;
   struct dsa_signature signature;
@@ -776,38 +838,90 @@ test_dsa(const struct dsa_public_key *pub,
   knuth_lfib_init(&lfib, 1111);
   
   sha1_update(&sha1, LDATA("The magic words are squeamish ossifrage"));
-  dsa_sign(pub, key,
-	   &lfib, (nettle_random_func *) knuth_lfib_random,
-	   &sha1, &signature);
-  
+  ASSERT (dsa_sha1_sign(pub, key,
+			&lfib, (nettle_random_func *) knuth_lfib_random,
+			&sha1, &signature));
+
   if (verbose)
     {
-      fprintf(stderr, "dsa signature: ");
+      fprintf(stderr, "dsa160 signature: ");
       mpz_out_str(stderr, 16, signature.r);
       fprintf(stderr, ", ");
       mpz_out_str(stderr, 16, signature.s);
       fprintf(stderr, "\n");
     }
 
-#if 0
-  if (mpz_cmp(signature, expected))
-    FAIL();
-#endif
+  if (expected)
+    if (mpz_cmp (signature.r, expected->r)
+	|| mpz_cmp (signature.s, expected->s))
+      FAIL();
   
   /* Try bad data */
-  if (DSA_VERIFY(pub, &sha1,
+  if (DSA_VERIFY(pub, sha1,
 		 "The magick words are squeamish ossifrage", &signature))
     FAIL();
 
   /* Try correct data */
-  if (!DSA_VERIFY(pub, &sha1,
+  if (!DSA_VERIFY(pub, sha1,
 		 "The magic words are squeamish ossifrage", &signature))
     FAIL();
 
   /* Try bad signature */
   mpz_togglebit(signature.r, 17);
 
-  if (DSA_VERIFY(pub, &sha1,
+  if (DSA_VERIFY(pub, sha1,
+		 "The magic words are squeamish ossifrage", &signature))
+    FAIL();
+
+  dsa_signature_clear(&signature);
+}
+
+void
+test_dsa256(const struct dsa_public_key *pub,
+	    const struct dsa_private_key *key,
+	    const struct dsa_signature *expected)
+{
+  struct sha256_ctx sha256;
+  struct dsa_signature signature;
+  struct knuth_lfib_ctx lfib;
+  
+  sha256_init(&sha256);
+  dsa_signature_init(&signature);
+  knuth_lfib_init(&lfib, 1111);
+  
+  sha256_update(&sha256, LDATA("The magic words are squeamish ossifrage"));
+  ASSERT (dsa_sha256_sign(pub, key,
+			&lfib, (nettle_random_func *) knuth_lfib_random,
+			&sha256, &signature));
+  
+  if (verbose)
+    {
+      fprintf(stderr, "dsa256 signature: ");
+      mpz_out_str(stderr, 16, signature.r);
+      fprintf(stderr, ", ");
+      mpz_out_str(stderr, 16, signature.s);
+      fprintf(stderr, "\n");
+    }
+
+  if (expected)
+    if (mpz_cmp (signature.r, expected->r)
+	|| mpz_cmp (signature.s, expected->s))
+      FAIL();
+  
+  /* Try bad data */
+  if (DSA_VERIFY(pub, sha256,
+		 "The magick words are squeamish ossifrage", &signature))
+    FAIL();
+
+  /* Try correct data */
+  if (!DSA_VERIFY(pub, sha256,
+		 "The magic words are squeamish ossifrage", &signature))
+    FAIL();
+
+  /* Try bad signature */
+  mpz_togglebit(signature.r, 17);
+
+  if (DSA_VERIFY(pub, sha256,
 		 "The magic words are squeamish ossifrage", &signature))
     FAIL();
 
@@ -816,14 +930,15 @@ test_dsa(const struct dsa_public_key *pub,
 
 void
 test_dsa_key(struct dsa_public_key *pub,
-	     struct dsa_private_key *key)
+	     struct dsa_private_key *key,
+	     unsigned q_size)
 {
   mpz_t t;
 
   mpz_init(t);
 
-  ASSERT(mpz_sizeinbase(pub->q, 2) == DSA_Q_BITS);
-  ASSERT(mpz_sizeinbase(pub->p, 2) >= DSA_MIN_P_BITS);
+  ASSERT(mpz_sizeinbase(pub->q, 2) == q_size);
+  ASSERT(mpz_sizeinbase(pub->p, 2) >= DSA_SHA1_MIN_P_BITS);
   
   ASSERT(mpz_probab_prime_p(pub->p, 10));
 
